"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const re = require("request-promise-native");
const url_1 = require("url");
class EtherscanAgent {
    constructor(cfg) {
        this.cfg = null;
        this.cfg = cfg;
    }
    doGet(params, path) {
        const base = `${this.cfg.schema}://${this.cfg.host}:${this.cfg.port}`;
        path = path || '/api';
        const url = new url_1.URL(path, base);
        if (params['module'] == null) {
            params['module'] = 'proxy';
        }
        Object.keys(params)
            .filter(key => params[key] != null)
            .forEach(key => url.searchParams.append(key, `${params[key]}`));
        return re(url.href, {
            method: 'GET',
            timeout: this.cfg.connectTimeoutInMills,
        });
    }
    /** Relative to Health Check */
    ping() {
        const params = {};
        return this.doGet(params, '/api/ping');
    }
    /** Relative to Block */
    getLatestBlockNum() {
        const params = {
            'action': 'eth_blockNumber',
        };
        return this.doGet(params);
    }
    getBlock(number, returnTransactionObjects) {
        const params = {
            action: 'eth_getBlockByNumber',
            tag: number,
            boolean: returnTransactionObjects,
        };
        return this.doGet(params);
    }
    /** Relative to Gas */
    gasPrice() {
        const params = {
            action: 'eth_gasPrice',
        };
        return this.doGet(params);
    }
    estimateGas(tx) {
        const params = {
            action: 'eth_estimateGas',
            from: tx.from,
            to: tx.to,
            value: tx.value,
            data: tx.data
        };
        return this.doGet(params);
    }
    /** Relative to Account */
    getBalance(address, defaultBlock) {
        const params = {
            module: 'account',
            action: 'balance',
            address: address,
            tag: defaultBlock,
        };
        return this.doGet(params);
    }
    getTransactionCount(address, defaultBlock) {
        const params = {
            action: 'eth_getTransactionCount',
            address: address,
            tag: defaultBlock || 'latest',
        };
        return this.doGet(params);
    }
    /** Relative to Transaction */
    callTransaction(tx, defaultBlock) {
        const params = {
            action: 'eth_call',
            from: tx.from,
            to: tx.to,
            data: tx.data,
            tag: defaultBlock,
        };
        return this.doGet(params);
    }
    getTransactionFromBlock(number, index) {
        const params = {
            action: 'eth_getTransactionByBlockNumberAndIndex',
            tag: typeof (number) == 'number' ? `0x${number.toString(16)}` : number,
            index: `0x${index.toString(16)}`,
        };
        return this.doGet(params);
    }
    sendSignedTransaction(txSigned) {
        return Promise.reject('unimplemented method');
    }
    getTransaction(txHash) {
        const params = {
            action: 'eth_getTransactionByHash',
            txhash: txHash,
        };
        return this.doGet(params);
    }
    getTransactionReceipt(txHash) {
        const params = {
            action: 'eth_getTransactionReceipt',
            txhash: txHash,
        };
        return this.doGet(params);
    }
}
exports.EtherscanAgent = EtherscanAgent;
